@@ -68,28 +68,34 @@
  *        Makefile, regnerate the prototypes, and recompile
  *        the libraries.  Look at the Makefile to see how I've
  *        included fhmtgen.1.c and fhmtgenlow.1.c.  These files
- *        provide the high-level interfaces for the hmt, and
- *        the low-level interfaces to do the actual work.
+ *        provide the single high-level interface for the hmt, and
+ *        the lower-level functions to do the actual work.
  *
  *    (4) In an application, you now use this interface.  Again
  *        for the example files generated, using integer "1":
  *
  *           PIX   *pixHMTDwa_1(PIX *pixd, PIX *pixs, const char *selname);
  *
- *              or
- *
- *           PIX   *pixFHMTGen_1(PIX *pixd, PIX *pixs, const char *selname);
- *
  *        where the selname is one of the set that were defined
  *        as the name field of sels.  This set is listed at the
  *        beginning of the file fhmtgen.1.c.
- *        As an example, see the file prog/fmtauto_reg.c, which
+ *
+ *        N.B. Although pixFHMTGen_1() is global, you must NOT
+ *        use it, because it assumes that 32 or 64 border pixels
+ *        have been added to each side, and it will crash without those
+ *        added pixels.
+ *
+ *        As an example, see the file prog/fhmtauto_reg.c, which
  *        verifies the correctness of the implementation by
  *        comparing the dwa result with that of full-image
  *        rasterops.
  * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config_auto.h>
+#endif  /* HAVE_CONFIG_H */
+
 #include <string.h>
 #include "allheaders.h"
 
@@ -99,7 +105,7 @@
 
 #define   PROTOARGS   "(l_uint32 *, l_int32, l_int32, l_int32, l_uint32 *, l_int32);"
 
-static const l_int32  L_BUF_SIZE = 512;
+#define L_BUF_SIZE 512
 
 static char * makeBarrelshiftString(l_int32 delx, l_int32 dely, l_int32 type);
 static SARRAY * sarrayMakeInnerLoopDWACode(SEL *sel, l_int32 nhits, l_int32 nmisses);
@@ -296,9 +302,9 @@ SARRAY  *sa1, *sa2, *sa3;
     str_doc1 = stringNew(bigbuf);
     sprintf(bigbuf, " *             PIX     *pixFHMTGen_%d()", fileindex);
     str_doc2 = stringNew(bigbuf);
-    sprintf(bigbuf, " *  pixHMTDwa_%d()", fileindex);
+    sprintf(bigbuf, " * \\brief  pixHMTDwa_%d()", fileindex);
     str_doc3 = stringNew(bigbuf);
-    sprintf(bigbuf, " *  pixFHMTGen_%d()", fileindex);
+    sprintf(bigbuf, " * \\brief  pixFHMTGen_%d()", fileindex);
     str_doc4 = stringNew(bigbuf);
     sprintf(bigbuf, "pixHMTDwa_%d(PIX         *pixd,", fileindex);
     str_def1 = stringNew(bigbuf);
@@ -585,7 +591,8 @@ SEL     *sel;
             }
         }
         if (nhits == 0) {
-            linestr = stringNew("    fprintf(stderr, \"Error in HMT: no hits in sel!\\n\");\n}\n\n");
+            linestr = stringNew("    "
+                "lept_stderr(\"Error in HMT: no hits in sel!\\n\");\n}\n\n");
             sarrayAddString(sa4, linestr, L_INSERT);
             continue;
         }
